﻿using Org.BouncyCastle.Asn1.Pkcs;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.OpenSsl;
using Org.BouncyCastle.Pkcs;
using Org.BouncyCastle.Security;
using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;
using Syncfusion.Pdf.Security;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace DigitalSignature
{
    class Program
    {
        static void Main(string[] args)
        {
            //Creates a new PDF document
            PdfDocument document = new PdfDocument();
        
            //Adds a new page
            PdfPageBase page = document.Pages.Add();
            PdfGraphics graphics = page.Graphics;

            Org.BouncyCastle.X509.X509CertificateParser cp = new Org.BouncyCastle.X509.X509CertificateParser();
            Org.BouncyCastle.X509.X509Certificate cert = cp.ReadCertificate(File.ReadAllBytes("../../Data/certificate.cer"));
            PemReader pmr = new PemReader(new StringReader(File.ReadAllText("../../Data/privateKey.pem")));
            AsymmetricCipherKeyPair KeyPair = (AsymmetricCipherKeyPair)pmr.ReadObject();

            Stream pfxFile = CreatePfxFile(cert, KeyPair.Private);

            //Creates a certificate instance from PFX file with private key
            PdfCertificate pdfCert = new PdfCertificate(pfxFile, "syncfusion");

            //Creates a digital signature
            PdfSignature signature = new PdfSignature(document, page, pdfCert, "Signature");           

            //Sets signature information
            signature.Bounds = new RectangleF(new PointF(0, 0), new SizeF(100,100));
            signature.ContactInfo = "johndoe@owned.us";
            signature.LocationInfo = "Honolulu, Hawaii";
            signature.Reason = "I am author of this document.";

            signature.Appearance.Normal.Graphics.DrawString("Hello World!!!", new PdfStandardFont(PdfFontFamily.Helvetica, 10), PdfBrushes.Black, new PointF(0, 0));

            //Saves and closes the document
            document.Save("Output.pdf");

            document.Close(true);

            System.Diagnostics.Process.Start("Output.pdf");
        }
        private static Stream CreatePfxFile(Org.BouncyCastle.X509.X509Certificate certificate, AsymmetricKeyParameter privateKey)
        {
            // Create certificate entry
            var certEntry = new X509CertificateEntry(certificate);
            string friendlyName = certificate.SubjectDN.ToString();

            // Get bytes of private key.
            PrivateKeyInfo keyInfo = PrivateKeyInfoFactory.CreatePrivateKeyInfo(privateKey);
            byte[] keyBytes = keyInfo.ToAsn1Object().GetEncoded();

            var builder = new Pkcs12StoreBuilder();
            builder.SetUseDerEncoding(true);
            var store = builder.Build();

            // create store entry
            store.SetKeyEntry("private", new AsymmetricKeyEntry(privateKey), new X509CertificateEntry[] { certEntry });

            MemoryStream stream = new MemoryStream();

            store.Save(stream, "syncfusion".ToCharArray(), new SecureRandom());
            return stream;
        }
    }
}
